aboutsummaryrefslogtreecommitdiff
path: root/src/nvim/eval.c
diff options
context:
space:
mode:
Diffstat (limited to 'src/nvim/eval.c')
-rw-r--r--src/nvim/eval.c29
1 files changed, 18 insertions, 11 deletions
diff --git a/src/nvim/eval.c b/src/nvim/eval.c
index 6774000ae4..1bddafbee3 100644
--- a/src/nvim/eval.c
+++ b/src/nvim/eval.c
@@ -8466,9 +8466,9 @@ static void f_executable(typval_T *argvars, typval_T *rettv, FunPtr fptr)
// Check in $PATH and also check directly if there is a directory name
rettv->vval.v_number = (
- os_can_exe((const char_u *)name, NULL, true)
+ os_can_exe(name, NULL, true)
|| (gettail_dir(name) != name
- && os_can_exe((const char_u *)name, NULL, false)));
+ && os_can_exe(name, NULL, false)));
}
typedef struct {
@@ -8573,12 +8573,12 @@ static void f_execute(typval_T *argvars, typval_T *rettv, FunPtr fptr)
static void f_exepath(typval_T *argvars, typval_T *rettv, FunPtr fptr)
{
const char *arg = tv_get_string(&argvars[0]);
- char_u *path = NULL;
+ char *path = NULL;
- (void)os_can_exe((const char_u *)arg, &path, true);
+ (void)os_can_exe(arg, &path, true);
rettv->v_type = VAR_STRING;
- rettv->vval.v_string = path;
+ rettv->vval.v_string = (char_u *)path;
}
/// Find a window: When using a Window ID in any tab page, when using a number
@@ -12084,10 +12084,11 @@ static void f_jobresize(typval_T *argvars, typval_T *rettv, FunPtr fptr)
/// @param[out] executable Returns `false` if argv[0] is not executable.
///
/// @returns Result of `shell_build_argv()` if `cmd_tv` is a String.
/// Else, string values of `cmd_tv` copied to a (char **) list.
static char **tv_to_argv(typval_T *cmd_tv, const char **cmd, bool *executable)
{
- if (cmd_tv->v_type == VAR_STRING) {
+ if (cmd_tv->v_type == VAR_STRING) { // String => "shell semantics".
const char *cmd_str = tv_get_string(cmd_tv);
if (cmd) {
*cmd = cmd_str;
@@ -12107,16 +12108,17 @@ static char **tv_to_argv(typval_T *cmd_tv, const char **cmd, bool *executable)
return NULL;
}
- const char *exe = tv_get_string_chk(TV_LIST_ITEM_TV(tv_list_first(argl)));
- if (!exe || !os_can_exe((const char_u *)exe, NULL, true)) {
- if (exe && executable) {
+ const char *arg0 = tv_get_string_chk(TV_LIST_ITEM_TV(tv_list_first(argl)));
+ char *exe_resolved = NULL;
+ if (!arg0 || !os_can_exe(arg0, &exe_resolved, true)) {
+ if (arg0 && executable) {
*executable = false;
}
return NULL;
}
if (cmd) {
- *cmd = exe;
+ *cmd = exe_resolved;
}
// Build the argument vector
@@ -12127,10 +12129,15 @@ static char **tv_to_argv(typval_T *cmd_tv, const char **cmd, bool *executable)
if (!a) {
// Did emsg in tv_get_string_chk; just deallocate argv.
shell_free_argv(argv);
+ xfree(exe_resolved);
return NULL;
}
argv[i++] = xstrdup(a);
});
+ // Replace argv[0] with absolute path. The only reason for this is to make
+ // $PATHEXT work on Windows with jobstart([…]). #9569
+ xfree(argv[0]);
+ argv[0] = exe_resolved;
return argv;
}
@@ -12084,10 +12084,11 @@ static void f_jobresize(typval_T *argvars, typval_T *rettv, FunPtr fptr)
///
/// @returns Result of `shell_build_argv()` if `cmd_tv` is a String.
-/// Else, string values of `cmd_tv` copied to a (char **) list.
+/// Else, string values of `cmd_tv` copied to a (char **) list with
+/// argv[0] resolved to full path ($PATHEXT-resolved on Windows).
static char **tv_to_argv(typval_T *cmd_tv, const char **cmd, bool *executable)
{